module Gdsapi
  module Methods
    class BaseMethod
      include ParseUtils
      attr_reader :requester

      # Constructor method for passing requester instance
      # @param [[Gdsapi::Requester]] - requester instance
      def initialize(requester)
        @requester = requester
      end

      ## Generic entry-point for action
      ## @param hash_styled `list` and `offset` args used for chunking requests
      ## @return - parsed results
      def call(**params)
        response_body = query params
        body = JSON.parse response_body
        parse body
      rescue JSON::ParserError
        raise MalformedGdsResponse, "Invalid JSON: #{response_body}"
      end

      protected
      # HTTP verb used to query GDS API for method
      # Defaults to :get
      # @return [Symbol] - symbolic alias for requester method corresponding to HTTP verb (i.e., :get or :post)
      def method
        :get
      end

      # GDS API path for method
      # It is required for method to specify path
      def parse(*args)
        raise NotImplementedError, 'Parsing is not allowed for abstract method'
      end

      # GDS API path for method
      # It is required for method to specify path
      def path
        raise NotImplementedError, 'Querying is not allowed for abstract method'
      end

      # Query routine to be executed while performing action
      # @param [Hash] args - args used for querying
      # @return [String] - response body string
      def query(args)
        params = transform_params(slice_params(args))
        response = requester.public_send(method, path, params: params)
        case response.status
          when (404)
            raise NotFoundError, "Not found on #{response.env.url}"
          when (401)
            raise UnathorizedError, "Invalid auth on #{response.env.url}"
          when (400..499)
            raise BadRequestError, "Bad request on #{response.env.url}: #{response.body.to_s}"
          when (500..599)
            raise GdsInternalError, "#{response.status} #{response.body.to_s}"
          when (300..399)
            raise GdsRedirectError, "#{response.status} #{response.headers['Location'].to_s}"
          else
            response.body
        end
      end


      # Transform params with accordance to requirements of API
      # @param [Hash[Symbol => Object]] filtered_params - passed params
      # @return [Hash[Symbol => Object]] transformed set of params
      def transform_params(filtered_params)
        filtered_params
      end

      # Select params which are allowed in querying for the method
      # @param [Hash[Symbol => Object]] params - passed params
      # @return [Hash[Symbol => Object]] params filtered using whitelist (see #permitted_params) and by presence (!nil?)
      def slice_params(params)
         params.select { |name, value| permitted_params.include?(name) && value }
      end

      # whitelist of permitted params
      # defaults to empty list
      # @return [Array[Symbol]] - params that are permitted for given instance of method
      def permitted_params
        []
      end

      # Editing geodata modes mapping (GDS -> output)
      # Used when dealing with temporal data (i.e. given timestamps)
      # U (upsert) - update or create record
      # D (delete) - delete record
      def modes
        {
          'U' => :upsert,
          'D' => :delete,
        }
      end
    end
  end
end
